#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import csv
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import datetime
from scipy.ndimage.filters import gaussian_filter1d
#matplotlib.use('Agg')
    
# Constants from SR104 resistor lid
ALPHA    = 0.089  # Alpha value at +23c
BETA     = -0.0031 # Beta value
R_DEV    = 0    # Deviation value for standard resistor
TEMP_DEV = 0.004  # Deviation value for RT
STEP     = 0.5    # Step to go thru temperature points in mode 2
STDREF   = 9998.0
#count = 0
fn_input = "test1_k7168d_scan_fx7_pretrim_tc_jun2019.dsv"


def show_r(temp, rsi):
    f = 0
    delta = temp - 23.0
    ppm = 0.0 + (ALPHA * delta) + (BETA * delta * delta)
    r = STDREF + ((ppm+R_DEV) * STDREF / 1000000.0)
    f = (temp * 9.0 / 5.0) + 32.0
    print ("temp = %5.2fC / %6.2fF ppm = %7.4f Rstd = %12.8f Ohms, %3.3f ppm\n" % (temp,f, R_DEV+ppm, r, (((r / rsi) - 1) *1e6 ) ) ),
    return r

def calc_ppmdev(temp, rsi, a, b):
    f = 0
    rdev = 0
    delta = temp - 23.0
    ppm = 0.0 + (a * delta) + (b * delta * delta)
    r = rsi + ((ppm+rdev) * rsi / 1000000.0)
    f = (temp * 9.0 / 5.0) + 32.0
    #print ("temp = %5.2fC / %6.2fF ppm = %7.4f Rstd = %12.8f Ohms, %3.3f ppm\n" % (temp,f, R_DEV+ppm, r, (((r / rsi) - 1) *1e6 ) ) ),
    return (((r / rsi) - 1) *1e6 )
    
print "\033[37;1m-- xDevs.com/EEVBlog RComp | SR104 correction calculator -- \n-- https://xdevs.com/review/sr104/  Rev.2 May 2019           --"
print "R deviation = %.4f ppm    T deviation = %.4f%%\n" % (R_DEV, TEMP_DEV),
print "Alpha = %.4f ppm/C       Beta = %.4f ppm/C/C\n" % (ALPHA, BETA)

#mode = int(raw_input("Enter 1 : calculate Rstd at Rrtd \r\nEnter 2 : table curve generate\r\nEnter 3 : Eat DSV-file to calculate alpha/beta/delta data\r\nEnter 4 : matplotlib sample data\r\nEnter 5 : Eat DCV and plot diag graph\r\n"))
mode = 5
if (mode == 1):
    rt = float(raw_input("Room Temp: "))
    rs = float(raw_input("Standard resistance: "))
    rs = STDREF + (R_DEV * STDREF / 1000000.0)
    print "Rt=%f  Rs=%f\n" % (rt, rs)
    r = (rt - rs) * 100.0 / STDREF
    r = r - TEMP_DEV
    r = r * 10.0
    temp = r + 23.0
    show_r(temp, STDREF)
elif (mode == 2):
    count = int(35 / STEP);
    for ix in range (0,count+1):
        temp = float(16 + ix * STEP)
        show_r(temp, STDREF)
elif (mode == 3):
    # DSV data parser to calculate alpha/beta values
    global fn_input
    fn_output = "tcr_math_output_dcv.txt"

    temp = []
    val = [[]]
    polyz = []
    for ival in range (0,8):
        val.append([])
    #val2 = []
    #val3 = []
    #val4 = []
    #val5 = []
    #val6 = []
    cnt = 0
    alpha = []
    beta = []
    gain = []
    tvalue = []
    c1 = []
    c2 = []
    sdev = []
    delta = []
    zerotctemp = []
    valzerotc = []
    cnx = []
    
    with open(fn_input) as csvfile:
        spamreader = csv.reader(csvfile, delimiter=';', quotechar='|')
        is_first = True
        for row in spamreader:
            cnt = cnt + 1
            if is_first:
                is_first = False
                continue
            #print ("%d" % cnt)
            if cnt <= 2:
                continue
            else:
                cnx.append(cnt - 3)
                temp.extend([float(row[13])])
                val[0].extend([float(row[1])]) #val1
                val[1].extend([float(row[2])]) #val2
                val[2].extend([float(row[3])]) #val3
                val[3].extend([float(row[4])]) #val4
                val[4].extend([float(row[5])]) #val5
                val[5].extend([float(row[6])]) #val6
                val[6].extend([float(row[7])]) #val7
                val[7].extend([float(row[8])]) #val8
            #corr.extend([(float(row[2]) + float(row[6]) ) / 2])
            #date.extend([row[0]])
    csvfile.close()
    
    tdelta = np.amax(temp) - np.amin(temp)
    reftemp = 23.000
    
    #print "Temperature min = %.3f , max = %.3f, delta = %.3f C" % (np.amin(temp), np.amax(temp), tdelta)
    #print "REF temp = %.3f C" % reftemp
    nominal = [10, 10, 10, 10, 10, 10, 10, 0, 0]
    print "\033[2J"
    with open(fn_output, 'a') as logfile:
        logfile.write ("\r\rFile = %s\r" % fn_input)
    logfile.close()
    
    for zval in range (0,8):
        polyz.append(np.polyfit(temp, val[zval], 2))
        c2.append(polyz[zval][0])
        c1.append(polyz[zval][1])
        gain.append(polyz[zval][2])
        tvalue.append(c2[zval] * reftemp*reftemp + c1[zval]*reftemp + gain[zval])
        alpha.append((c1[zval] + 2*reftemp*c2[zval]) / tvalue[zval]*1e6 )
        beta.append(c2[zval] / tvalue[zval]*1e6 )
        delta.append((tvalue[zval] - nominal[zval]) / nominal[zval] * 1e6)
        zerotctemp.append(-alpha[zval] / 2 / beta[zval] + reftemp)
        valzerotc.append(nominal[zval] + (zerotctemp[zval]) * c1[zval] + (zerotctemp[zval])*2*c2[zval])
        sdev.append(np.std(val[zval], axis=0))
        
        if (gain[zval] != 0):
            print "\033[37;1m\r\nNominal       :", nominal[zval]
            print ("Column number : %d " % zval), polyz[zval], "Total samples = %d" % len(val[zval])
            print ("c1            : %e" % c1[zval])
            print ("c2            : %e" % c2[zval])
            print ("g             : %e" % gain[zval])
            print ("Value T23     : %.8f" % (tvalue[zval]))
            print ("Sdev          : %.6e" % (sdev[zval]))
            print ("\033[36;1mAlpha T23     : %.4f ppm/K       " % (alpha[zval])),
            print ("Beta      : %.4f ppm/K/2                " % (beta[zval])),
            print ("Delta T23 : %.3f ppm         \033[39;1m" % (delta[zval]))
            print ("\033[35;1mZero TC       : %.3f C \033[39;0m" % (zerotctemp[zval])),
            print ("\033[32;1m          Delta ZeroTC : %.8f\033[39;0m" % (valzerotc[zval]))
            
            with open(fn_output, 'a') as logfile:
                logfile.write ("\rColumn number : %d\r" % zval), polyz[zval], "Total samples = %d" % len(val[zval])
                logfile.write ("Nominal       : %.4f\r" % nominal[zval])
                logfile.write ("c1            : %e\r" % c1[zval])
                logfile.write ("c2            : %e\r" % c2[zval])
                logfile.write ("g             : %e\r" % gain[zval])
                logfile.write ("Value T23     : %.6f\r" % (tvalue[zval]))
                logfile.write ("Alpha T23     : %.4f ppm/K     \r" % (alpha[zval])),
                logfile.write ("Beta          : %.4f ppm/K/2   \r" % (beta[zval])),
                logfile.write ("Delta T23     : %.3f ppm       \r" % (delta[zval]))
                logfile.write ("Zero TC       : %.3f C \r" % (zerotctemp[zval])),
                logfile.write ("Delta ZeroTC  : %.6f   \r" % (valzerotc[zval]))
            logfile.close()
            
elif (mode == 4):
    # Do matplotlib test
    count = int(35 / STEP);
    ppmvals = []
    ppmvals2 = []
    ppmvals3 = []
    ppmvals4 = []
    ppmvals5 = []
    ppmvals6 = []
    ppmvals7 = []
    ppmvals8 = []
    ppmvals9 = []
    ppmvalsa = []
    ppmvalsb = []
    ppmvalsc = []
    ppmvalsd = []
    tempvals = []
    w = 20
    h = 16
    
    for ix in range (0,count+1):
        temp = float(16 + ix * STEP)
        ppmvals.append(calc_ppmdev (temp,   1, 2.1925, -0.0259))  # Res data 1
        ppmvals2.append(calc_ppmdev(temp,   1, 2.5640, -0.0242))  # Res data 2
        ppmvals3.append(calc_ppmdev(temp,   1,-0.8158, -0.0197)) # Res data 3
        ppmvals4.append(calc_ppmdev(temp,   1, 1.0558, -0.0257)) # Res data 4
        ppmvals5.append(calc_ppmdev(temp,   1, 1.2296, -0.0256))  # Res data 5
        ppmvals6.append(calc_ppmdev(temp,   1,-1.6999, -0.0222)) # Res data 6
        ppmvals7.append(calc_ppmdev(temp,   1, 0.9006, -0.0258)) # Res data 7
        ppmvals8.append(calc_ppmdev(temp,   1,-0.2210, -0.0229)) # Res data 7
        ppmvals9.append(calc_ppmdev(temp,   1, 0.01117, 0.00023)) # Res data 7
        #ppmvalsa.append(calc_ppmdev(temp,   1,-1.6999, -0.0222)) # Res data 6
        #ppmvalsb.append(calc_ppmdev(temp,   1, 0.9006, -0.0258)) # Res data 7
        #ppmvalsc.append(calc_ppmdev(temp,   1,-0.2210, -0.0229)) # Res data 7
        #ppmvalsd.append(calc_ppmdev(temp,   1, 1.8569, -0.0239)) # Res data 7
        
        tempvals.append(temp)
    #print ppmvals
    #print tempvals
    
    fig,ax = plt.subplots()
    fig.set_size_inches(w,h)
    plt.tight_layout(rect=[0.02, 0.02, 1.0, 0.98], w_pad=0, h_pad=0.5)                                # Reduce margins around the graph
        
    ax.plot(tempvals, ppmvals,  '.-', label='RESA', linewidth=1.0)
    ax.plot(tempvals, ppmvals2, '.-', label='RESB', linewidth=1.0)
    ax.plot(tempvals, ppmvals3, '.-', label='RESC', linewidth=1.0)
    ax.plot(tempvals, ppmvals4, '.-', label='RESA', linewidth=1.0)
    ax.plot(tempvals, ppmvals5, '.-', label='RESB', linewidth=1.0)
    ax.plot(tempvals, ppmvals6, '-.', label='RESC', linewidth=1.0)
    ax.plot(tempvals, ppmvals7, '.-', label='RESA', linewidth=1.0)
    ax.plot(tempvals, ppmvals8, '.-', label='RESA', linewidth=1.0)
    ax.plot(tempvals, ppmvals9, '-', label='RESA', linewidth=4.0)
    #ax.plot(tempvals, ppmvalsa, '-.', label='RESC', linewidth=1.0)
    #ax.plot(tempvals, ppmvalsb, '.-', label='RESA', linewidth=1.0)
    #ax.plot(tempvals, ppmvalsc, '.-', label='RESA', linewidth=1.0)
    #ax.plot(tempvals, ppmvalsd, '.-', label='RESA', linewidth=2.0)
    ax.legend(["JRL 1 Ohm, #1", "JRL 1 Ohm, #2", "JRL 1 Ohm, #3", "JRL 1 Ohm, #4", 
               "JRL 1 Ohm, #5", "JRL 1 Ohm, #6", "JRL 1 Ohm, #7", "JRL 1 Ohm, #8", 
               "Fluke SL935 1R", "d", "JRL 1 Ohm, #11", "JRL 1 Ohm, #12"])
    start, end = ax.get_ylim()
    startx = 16
    endx = 51
    axis_y_step = 2
    axis_x_step = 1
    ax.yaxis.set_ticks(np.arange(start, end, axis_y_step))
    ax.xaxis.set_ticks(np.arange(startx, endx, axis_x_step))
    
    #plt.ylim(-2,2)
    plot_out_filename = "test.png"
    fig.savefig(plot_out_filename, facecolor=fig.get_facecolor(), transparent=False)
    plt.show()
    
elif (mode == 5):
    # DSV data parser to calculate alpha/beta values
    fn_input = "CalClub_cmtest.csv"
    fn_output = "tcr_math_output_ral.txt"

    temp = []
    val = [[]]
    ral = [[]]
    polyz = []
    for ival in range (0,17):
        val.append([])
    for ival in range (0,17):
        ral.append([])
    #val2 = []
    #val3 = []
    #val4 = []
    #val5 = []
    #val6 = []
    cnt = 0
    alpha = []
    beta = []
    gain = []
    tvalue = []
    c1 = []
    c2 = []
    delta = []
    zerotctemp = []
    valzerotc = []
    cnx = []
    date = []
    chop_samples = 25
    
    num_lines = sum(1 for line in open(fn_input))
    for dval in range (0,num_lines-chop_samples):
        date.append(" ")
    vref = [7.045641040e+00,9.9999568e+00,7.158675987e+00,9.983004444e+00,7.136260692e+00,9.962210853e+00,7.151827323e+00,9.981525961e+00,7.108384358e+00,9.923200475e+00,7.156064144e+00,9.999977018e+00,7.160140665e+00,1.000038978e+01,7.162542902e+00,9.996671352e+00]
    
    
    with open(fn_input) as csvfile:
        spamreader = csv.reader(csvfile, delimiter=';', quotechar='|')
        is_first = True
        dcount = 0
        for row in spamreader:
            cnt = cnt + 1
            dcount = dcount + 1
            if is_first:
                is_first = False
                continue
            #print ("%d" % cnt)
            if cnt <= chop_samples:
                continue
            else:
                cnx.append(cnt - (chop_samples+1))
                temp.extend([float(row[13])])
                date[dcount - (chop_samples+1)] = datetime.datetime.strptime (row[0],"%d/%m/%Y-%H:%M:%S")
                #date.extend(row[0])#.strftime('%d/%m/%y-%h:%m:%s')
                val[0].extend([ ( (float(row[1]) / vref[0]) - 1) * 1e6 ]) #val1
                val[1].extend([ ( (float(row[7]) / vref[1]) - 1) * 1e6 ]) #val2
                #val[2].extend([ ( (float(row[3]) / vref[2]) - 1) * 1e6 ]) #val3
                #val[3].extend([ ( (float(row[4]) / vref[3]) - 1) * 1e6 ]) #val4
                
                
                ral[0].extend([  float(row[1]) ]) #val1
                ral[1].extend([  float(row[7]) ]) #val2
                #ral[2].extend([  float(row[3]) ]) #val3
                #ral[3].extend([  float(row[4]) ]) #val4
                #ral[4].extend([  float(row[5]) ]) #val5
                #ral[5].extend([  float(row[6]) ]) #val6
                #ral[6].extend([  float(row[7]) ]) #val6
                #ral[7].extend([  float(row[8]) ]) #val6
                #ral[8].extend([  float(row[9]) ]) #val6
                #ral[9].extend([  float(row[10]) ]) #val6
                #ral[10].extend([ float(row[11]) ]) #val6
                #ral[11].extend([ float(row[12]) ]) #val6
                #ral[12].extend([ float(row[13]) ]) #val6
                #ral[13].extend([ float(row[14]) ]) #val6
                #ral[14].extend([ float(row[15]) ]) #val6
                #ral[15].extend([ float(row[16]) ]) #val6
                
                val[16].extend([0.0]) #val8
            #corr.extend([(float(row[2]) + float(row[6]) ) / 2])
            #date.extend([row[0]])
    csvfile.close()
    
    tdelta = np.amax(temp) - np.amin(temp)
    reftemp = 23.00
    nominal = []
    median = []
   
    #print "Temperature min = %.3f , max = %.3f, delta = %.3f C" % (np.amin(temp), np.amax(temp), tdelta)
    #print "REF temp = %.3f C" % reftemp
        
    print "\033[2J"
    with open(fn_output, 'a') as logfile:
        logfile.write ("\r\rFile = %s\r" % fn_input)
    logfile.close()
    
    for zval in range (0,2):
        #nominal = [9.999957591e+00, 9.999963348e+00, 0, 0, 1.000001420e+01, 9.999958800e+00, 9.515830000e-01, 0, 0]
        nominal.append(np.mean(ral[zval][:30]))
        polyz.append(np.polyfit(temp, ral[zval], 2))
        c2.append(polyz[zval][0])
        c1.append(polyz[zval][1])
        gain.append(polyz[zval][2])
        tvalue.append(c2[zval] * reftemp*reftemp + c1[zval]*reftemp + gain[zval])
        alpha.append((c1[zval] + 2*reftemp*c2[zval]) / tvalue[zval]*1e6 )
        beta.append(c2[zval] / tvalue[zval]*1e6 )
        delta.append((tvalue[zval] - nominal[zval]) / nominal[zval] * 1e6)
        zerotctemp.append(-alpha[zval] / 2 / beta[zval] + reftemp)
        valzerotc.append(nominal[zval] + (zerotctemp[zval]) * c1[zval] + (zerotctemp[zval])*2*c2[zval])
        median.append(np.median(ral[zval]))
        
        if (gain[zval] != 0):
            print "\033[37;1m\r\nNominal       :", nominal[zval]
            print ("Median        : %.8f" % median[zval])
            print ("Column number : %d " % zval), polyz[zval], "Total samples = %d" % len(ral[zval])
            print ("c1            : %e" % c1[zval])
            print ("c2            : %e" % c2[zval])
            print ("g             : %e" % gain[zval])
            print ("Value T23     : %.6f" % (tvalue[zval]))
            print ("\033[36;1mAlpha T23     : %.4f ppm/K       " % (alpha[zval])),
            print ("Beta      : %.4f ppm/K/2                " % (beta[zval])),
            print ("Delta T23 : %.3f ppm         \033[39;1m" % (delta[zval]))
            print ("\033[35;1mZero TC       : %.3f C \033[39;0m" % (zerotctemp[zval])),
            print ("\033[32;1m          Delta ZeroTC : %.6f\033[39;0m" % (valzerotc[zval]))
            
            with open(fn_output, 'a') as logfile:
                logfile.write ("\rColumn number : %d\r" % zval), polyz[zval], "Total samples = %d" % len(ral[zval])
                logfile.write ("Nominal       : %.8f\r" % nominal[zval])
                logfile.write ("c1            : %e\r" % c1[zval])
                logfile.write ("c2            : %e\r" % c2[zval])
                logfile.write ("g             : %e\r" % gain[zval])
                logfile.write ("Value T23     : %.8f\r" % (tvalue[zval]))
                logfile.write ("Alpha T23     : %.4f ppm/K     \r" % (alpha[zval])),
                logfile.write ("Beta          : %.4f ppm/K/2   \r" % (beta[zval])),
                logfile.write ("Delta T23     : %.3f ppm       \r" % (delta[zval]))
                logfile.write ("Zero TC       : %.3f C \r" % (zerotctemp[zval])),
                logfile.write ("Delta ZeroTC  : %.8f   \r" % (valzerotc[zval]))
            logfile.close()
    w = 20
    h = 16
    fig,ax = plt.subplots()
    fig.set_size_inches(w,h)
    plt.tight_layout(rect=[0.03, 0.03, 0.96, 0.98], w_pad=0.4, h_pad=0.4)                                # Reduce margins around the graph
        
    plt.xlabel('Date', fontsize=18)
    plt.ylabel('Deviation from ref, ppm', fontsize=18) #$\mu$V
    
    #new_x = dates.num2date(date)
    
    avgpower = 16
    
    val1aver = gaussian_filter1d(val[0], sigma=avgpower)
    val2aver = gaussian_filter1d(val[1], sigma=avgpower)
    #val3aver = gaussian_filter1d(val[2], sigma=avgpower)
    #val4aver = gaussian_filter1d(val[3], sigma=avgpower)
    #val5aver = gaussian_filter1d(val[4], sigma=avgpower)
    #val6aver = gaussian_filter1d(val[5], sigma=avgpower)
    #val7aver = gaussian_filter1d(val[6], sigma=avgpower)
    #val8aver = gaussian_filter1d(val[7], sigma=avgpower)
   
    temp2aver = gaussian_filter1d(temp, sigma=8)
    
    colors = ['#e6194b', '#f58231', '#1f5119', '#3cb44b', '#008080', '#4363d8', '#000076', '#911eb4', '#f032e6', '#00a68c', '#664433', '#1f38e3', '#ff0000', '#ec18a0', '#dbb201', '#1c2b33', '#003355', '#fc007f']
    
    avglinestype = 'solid'
    avglinestypen = 'none'
    
    plt.title('Scan USAC outputs (CM 3458A on PX, CM 8508A on FX)', fontsize=22)
    ax.plot(date, val1aver, '-', label='CH1',  linewidth=3.0,            color=colors[0], linestyle=avglinestype, marker='.', ms=1)
    ax.plot(date, val2aver, '-', label='CH2',  linewidth=3.0,            color=colors[1], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val3aver, '-', label='CH3',  linewidth=3.0,            color=colors[2], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val4aver, '-', label='CH4',  linewidth=3.0,            color=colors[3], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val5aver, '-', label='CH5',  linewidth=3.0,            color=colors[4], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val6aver, '-', label='CH6',  linewidth=3.0,            color=colors[5], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val7aver, '-', label='CH7',  linewidth=3.0,            color=colors[6], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val8aver, '-', label='CH8',  linewidth=3.0,            color=colors[7], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val9aver, '-', label='CH1',  linewidth=3.0,            color=colors[8], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val10aver, '-', label='CH2', linewidth=3.0,           color=colors[9], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val11aver, '-', label='CH3', linewidth=3.0,           color=colors[10], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val12aver, '-', label='CH4', linewidth=3.0,           color=colors[11], linestyle=avglinestype)#, marker='.', ms=1)
    #ax.plot(temp, val13aver, '-', label='CH5', linewidth=3.0,           color=colors[12], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val14aver, '-', label='CH6', linewidth=3.0,           color=colors[13], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val15aver, '-', label='CH7', linewidth=3.0,           color=colors[14], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(temp, val16aver, '-', label='CH8', linewidth=3.0,           color=colors[15], linestyle=avglinestype, marker='.', ms=1)
    
    
    ax.plot(date, val[0], '-'  , label='CH1', linewidth=0.2,  alpha=0.6, color=colors[0] )#, linestyle='none', marker='.', ms=6)
    ax.plot(date, val[1], '-'  , label='CH2', linewidth=0.2,  alpha=0.6, color=colors[1] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[2], '-'  , label='CH3', linewidth=0.2,  alpha=0.6, color=colors[2] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[3], '-'  , label='CH4', linewidth=0.2,  alpha=0.6, color=colors[3] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[4], '-'  , label='CH5', linewidth=0.2,  alpha=0.6, color=colors[4] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[5], '-'  , label='CH6', linewidth=0.2,  alpha=0.6, color=colors[5] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[6], '-'  , label='CH7', linewidth=0.2,  alpha=0.6, color=colors[6] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[7], '-'  , label='CH8', linewidth=0.2,  alpha=0.6, color=colors[7] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[8], '-'  , label='CH1', linewidth=0.2,  alpha=0.6, color=colors[8] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[9], '-'  , label='CH2', linewidth=0.2,  alpha=0.6, color=colors[9] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[10], '-'  , label='CH3', linewidth=1.0, alpha=0.6, color=colors[10])#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[11], '-'  , label='CH4', linewidth=3.0, alpha=0.6, color=colors[11])#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[12], '-'  , label='CH5', linewidth=1.0, alpha=0.6, color=colors[12])#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[13], '-'  , label='CH6', linewidth=1.0, alpha=0.6, color=colors[13])#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[14], '-'  , label='CH7', linewidth=1.0, alpha=0.6, color=colors[14])#, linestyle='none', marker='.', ms=6)
    #ax.plot(temp, val[15], '-'  , label='CH8', linewidth=1.0, alpha=0.6, color=colors[15])#, linestyle='none', marker='.', ms=6)
    
    
    #ax.plot(date, temp, '--', label='TEMP', linewidth=2.0, color='#00aa00', linestyle='solid', marker='.', ms=1)
    
    start, end = ax.get_ylim()
    startx, endx = ax.get_xlim()
    axis_y_step = 0.01
    #axis_x_step = 1
    ex = np.arange(1,2000)
    #ax.yaxis.set_ticks(np.arange(start, end)) # axis_y_step))
    ax.yaxis.set_ticks(np.arange(-1.2, 1.1, 0.05)) # axis_y_step))
    #ax.xaxis.set_ticks(np.arange(17.5, 41.5, 0.5)) # axis_y_step))
    #ax.xaxis.set_ticks(np.arange(1, 20000, 120)) # axis_y_step))
    #ax.xaxis.set_ticks(np.arange(startx, endx)) #, axis_x_step))
    
    ax2 = ax.twinx()  # instantiate a second axes that shares the same x-axis
    ax2.set_ylabel('Ambient temp, C', fontsize=18)  # we already handled the x-label with ax1
    ax2.plot(date, temp2aver, '-.', label='TEMP?', linewidth=1.0, color='#003300', linestyle='solid', marker='.', ms=2)
    ax2.tick_params(axis='y')
    legendlabel = ["7V PX 3458A CM ", "10V FX 8508A CM "]# "7.15867598 CH3 ", "9.98300444 CH4 ", "7.13626069 CH5 ", "9.96221085 CH6 ", "7.15182732 CH7 ", "9.98152596 CH8 ", "7.10838435 CH9 ", "9.92320047 CH10", "7.15606414 CH11", "FX 792X", "7.16014066 CH13", "1.00003897 CH14", "7.16254290 CH15", "9.99667135 CH16", "Zero Ref"]
    ax.legend(legendlabel, loc=4, fancybox=True, framealpha=1, borderpad=1, markerscale=10)
    ax.grid(True, color='#cccccc')
    ax2.yaxis.set_ticks(np.arange(20, 28, 0.5)) # axis_y_step))
    ax.xaxis.set_major_formatter(mdates.DateFormatter("%b %d, %H:%M:%S"))
    ax.xaxis.set_minor_formatter(mdates.DateFormatter("%b %d, %H:%M:%S"))

    vmed = []
    vmin = []
    vmax = []
    sdev = []
    tmin = np.amin(temp)
    tmax = np.amax(temp)
    temppp = tmax - 18
    tempco = []
    
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        #label.set_fontname('Arial')
        label.set_fontsize(13)
    
    for stats in range (0, 2):
        vmed.extend([0])
        vmin.extend([0])
        vmax.extend([0])
        sdev.extend([0])
        tempco.extend([0])
        
    for stats in range (0, 2):
        vmed[stats] = np.median(val[stats])
        vmin[stats] = np.amin(val[stats])# - vstep
        vmax[stats] = np.amax(val[stats])# + vstep
        sdev[stats] = np.std(val[stats]) # SDEV
        tempco[stats] = ((vmax[stats] - vmin[stats]) / temppp) 
        if ((stats % 2) == 0):
            bgclr = "#ffffff"
        else:
            bgclr = "#eeeeff"
            
        plt.text(0.01 + (stats * 0.15), 0.14, 'Max[%d] : %.3f ppm' % (stats, vmax[stats]),
            horizontalalignment='left',
            verticalalignment='top', backgroundcolor=bgclr,
            transform = ax.transAxes, size=11, color=colors[stats])
        plt.text(0.01 + (stats * 0.15), 0.11, 'Min[%d] : %.3f ppm' % (stats, vmin[stats]),
            horizontalalignment='left',
            verticalalignment='top', backgroundcolor=bgclr,
            transform = ax.transAxes, size=11, color=colors[stats])
        plt.text(0.01 + (stats * 0.15), 0.17,'Median[%d] : %.3f ppm' % (stats, vmed[stats]),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=12, color=colors[stats])
        plt.text(0.01 + (stats * 0.15), 0.2,'Sdev[%d] : %.3f ppm' % (stats, sdev[stats]),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=12, color=colors[stats])
        plt.text(0.01 + (stats * 0.15), 0.23, '%s' % (legendlabel[stats]),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=14, color=colors[stats])
        plt.text(0.01 + (stats * 0.15), 0.08, 'Alpha : %.3f ppm/K' % ( alpha[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=12, color=colors[stats])
        plt.text(0.01 + (stats * 0.15), 0.05, 'Beta : %.3f ppm/K2' % ( beta[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=12, color=colors[stats])
        plt.text(0.01 + (stats * 0.15), 0.03, 'Zero TC : %.3f C' % ( zerotctemp[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=12, color=colors[stats])
        
    
    smax = len(val[0])
    plt.text(0.07, -0.05,'xDevs.com | https://xdevs.com/article/teckit',
        horizontalalignment='center',
        verticalalignment='center',
        transform = ax.transAxes, size=12)
    plt.text(0.2, -0.04,'Samples : %d' % smax,
        horizontalalignment='left',
        verticalalignment='top',
        transform = ax.transAxes, size=12)
    
    #plt.ylim(-2,2)
    plot_out_filename = "test_precal-scan_fx.png"
    fig.savefig(plot_out_filename, facecolor=fig.get_facecolor(), transparent=False)
    plt.show()